#include <stdlib.h>
#include <stdint.h>
#include "proto.h"

/*
 * Binary protocol
 */
enum {
	TYPE_NULL = 0,	/* no length, just id, type */
	TYPE_BOOLEAN,	/* no length, id, type, value  */
	TYPE_NUMBER,	/* id, type, length, value */
	TYPE_STRING,	/* id, type, length, value */
};

static struct proto_type proto_type_map[] = {
	{ VAR_NONE,      TYPE_NULL,   0,                FLAGS_BOTH },
	{ VAR_REQUESTID, TYPE_NUMBER, 4,                FLAGS_BOTH },		/* Response/Request: Request ID */
	{ VAR_VERSION,   TYPE_NUMBER, 4,                FLAGS_REQUEST },	/* Request: Version of protocol */
	{ VAR_LOOKUP,    TYPE_STRING, MAX_KEY_SIZE,     FLAGS_REQUEST },	/* Request: Value to look up */
	{ VAR_COST,      TYPE_NUMBER, 4,                FLAGS_RESPONSE },	/* Response: Cost of route */
	{ VAR_GATEWAY,   TYPE_STRING, MAX_GATEWAY_SIZE, FLAGS_RESPONSE },	/* Response: Gateway of route */
	{ VAR_STATUS,    TYPE_NUMBER, 4,                FLAGS_RESPONSE },	/* Response: Status Code */
	{ VAR_INVALID,   TYPE_NUMBER, 4,                FLAGS_RESPONSE },	/* Response: Invalid field */
	{ VAR_DURATION,  TYPE_NUMBER, 8,                FLAGS_RESPONSE },	/* Response: Raw query time */
};

struct proto_type *lookup_type( int id )
{
	for( int x = 0; x < (sizeof(proto_type_map) / sizeof(struct proto_type)); x++ ) {
		if( proto_type_map[x].id == id )
			return &proto_type_map[x];
	}
	return NULL;
}
