#ifndef __NET_H__
#define __NET_H__

#include <netinet/in.h>
#include <arpa/inet.h>

struct netaddr {
	union {
		struct sockaddr		sa;
		struct sockaddr_in	ip4;
		struct sockaddr_in6	ip6;
	} addr;
};

void netaddr_init( struct netaddr *na );
struct netaddr *netaddr_create( const struct sockaddr *sa );
int netaddr_destroy( struct netaddr **na );

int netaddr_is_ipv4( const struct netaddr *na );
int netaddr_is_ipv6( const struct netaddr *na );

int netaddr_get_family( const struct netaddr *na );
int netaddr_set_family( struct netaddr *na, const int family );

unsigned short netaddr_get_port( const struct netaddr *na );
int netaddr_set_port( struct netaddr *na, const unsigned short port );

int netaddr_set( struct netaddr *na, const struct sockaddr *sa );
int netaddr_get( const struct netaddr *na, struct sockaddr *sa, socklen_t *salen );

const char * netaddr_ntop( const struct netaddr *na, char *buf, socklen_t buflen );
int netaddr_pton( struct netaddr *na, const char *buf );

#endif
