/*
 * Julienne Walker's Red-Black tree implementation
 *
 * URL: http://eternallyconfuzzled.com/tuts/datastructures/jsw_tut_rbtree.aspx
 * License: Public-Domain
 */
#ifndef _RB_TREE_H_
#define _RB_TREE_H_

struct rb_node;

struct rb_tree {
	/* add locking here... */
	struct rb_node *root;
};

/**
 * rb_find
 * \brief	Lookup key in r-b tree
 * \param[in]	tree	Pointer to rb_tree struct
 * \param[in]	key	Key to look up
 * \return	Stored value or NULL if not found
 */
void *rb_find( struct rb_tree *tree, uint32_t key );

/**
 * rb_insert
 * \brief	Insert key into r-b tree
 * \param[in]	tree	Pointer to rb_tree struct
 * \param[in]	key	Key to store
 * \param[in]	value	(Pointer to) Associated value
 * \return	0 on success, < 0 otherwise
 */
int rb_insert( struct rb_tree *tree, uint32_t key, void *value );

/**
 * rb_remove
 * \brief	Remove key from r-b tree
 * \param[in]	tree	Pointer to rb_tree struct
 * \param[in]	key	Key to remove
 * \return	0 on success, < 0 otherwise
 */
int rb_remove( struct rb_tree *tree, uint32_t key );


#endif
